/*******************************************************************************
 * File name: Burner.h
 * Description: Flash Burner
 * Author: Shiva
 * Revision history:
 * 2010/10/01	Create this file.
 * ****************************************************************************/
#ifndef __Burner__
#define __Burner__

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <signal.h>
#include <sys/types.h>
#include <getopt.h>

#ifdef __MINGW32__
#include <winsock2.h>
#undef socklen_t
#define socklen_t int
#define sleep(t) _sleep(t * 1000)
#else
#include <sys/socket.h>
#include <netinet/in.h>
#include <netinet/tcp.h>
#include <netdb.h>
#include <arpa/inet.h>
#endif

#define RESET_TARGET (0x3a)
#define RESET_HOLD   (0x3b)
#define RESET_AICE   (0x3c)
#define HOLD_CORE    (0x1d)

#define MX_Type     (0x01)
#define SST_Type    (0x02)
#define Intel_Type  (0x03)
#define AMD_Type    (0x04)

#define FLASH_SINGLE (0x00)
#define FLASH_DOUBLE (0x01)
#define FLASH_FOUR   (0x02)

#define BLK_SIZE	0x40000
#define FLASH_SIZE  	0x2000000

/* type define */
typedef unsigned long long      UINT64;
typedef long long                       INT64;
typedef unsigned int            UINT32;
typedef int                                     INT32;
typedef unsigned short          UINT16;
typedef short                           INT16;
typedef unsigned char           UINT8;
typedef char                            INT8;

/* read/write API */
/* use these three API read/write target memory */
int outw(unsigned int address, unsigned int data);
unsigned int inw(unsigned int address);
int fastin(unsigned int address, unsigned int size, char * buffer);

/* Flash programming function */
void Flash_ReadID(unsigned flash_type, unsigned base, unsigned BusWidth, unsigned* DDI1, unsigned* DDI2);
void Flash_Reset(unsigned base, unsigned BusWidth);
void Flash_ChipErase(unsigned flash_type, unsigned base, unsigned BusWidth);
void Flash_ProgramByte(unsigned flash_type, unsigned base, unsigned address, UINT8 data);
void Flash_ProgramHalfWord(unsigned flash_type, unsigned base, unsigned address, UINT16 data);
void Flash_ProgramWord(unsigned flash_type, unsigned base, volatile unsigned address, volatile unsigned data);
void Flash_ProgramMultiWord(unsigned flash_type, unsigned base, volatile unsigned address, unsigned char *buffer, unsigned size);
void Check_Toggle_Ready (UINT8  *Dst);
void Check_Toggle_ReadyHalfWord(UINT16  *Dst);
void Check_Toggle_ReadyWord(unsigned  *Dst);

int Burn_image(unsigned flash_type, unsigned int address, unsigned char* start, unsigned int size, unsigned int base);
int Verification(unsigned int address, unsigned char* start, unsigned int size);
int Record_content(unsigned int address,unsigned char* buffer, unsigned int size);

unsigned int usage(void);
void terminate(void);
void handle_int(int signo);
int send_cmd(char cmd);

/* option structure */
static struct option long_option[] = 
{
  {"h", no_argument, 0, 'h'},
  {"help", no_argument, 0, 'h'},
  {"preserve", no_argument, 0, 'e'},
  {"reset-target", no_argument, 0, 't'},
  {"reset-hold", no_argument, 0, 'o'},
  {"target", required_argument, 0, 'x'},
  {"lock", no_argument, 0, 'z'},
  {"unlock", no_argument, 0, 'y'},
  {"verify", no_argument, 0, 'v'},
  {"source", no_argument, 0, 's'},
  {"version", no_argument, 0, 'V'},
  {"fast", no_argument, 0, 'g'},
  {"erase-all", no_argument, 0, 'u'},
  {"reset-hold", no_argument, 0, 'o'},
  {"log", required_argument, 0, 'l'},
  {"ip", required_argument, 0, 'k'},
  {"p", required_argument, 0, 'p'},
  {"port", required_argument, 0, 'p'},
  {"a", required_argument, 0, 'a'},
  {"addr", required_argument, 0, 'a'},
  {"f", required_argument, 0, 'f'},
  {"flash", required_argument, 0, 'f'},
  {"b", required_argument, 0, 'b'},
  {"base", required_argument, 0, 'b'},
  {"i", required_argument, 0, 'i'},
  {"image", required_argument, 0, 'i'}
};
#endif

